/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at
 * https://glassfish.dev.java.net/public/CDDLv1.0.html.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at https://glassfish.dev.java.net/public/CDDLv1.0.html.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems Inc. All Rights Reserved
 */

/*
 * $Id: DirectoryUtil.java,v 1.2 2006/04/13 01:31:54 ofung Exp $
 */

package com.sun.xml.rpc.processor.util;

import java.io.File;

import com.sun.xml.rpc.processor.generator.GeneratorException;
import com.sun.xml.rpc.util.ClassNameInfo;

/**
 * Util provides static utility methods used by other wscompile classes.
 *
 * @author JAX-RPC Development Team
 */
public class DirectoryUtil  {
    
    public static File getOutputDirectoryFor(String theClass,
        File rootDir, ProcessorEnvironment env) throws GeneratorException {
            
        File outputDir = null;
        String qualifiedClassName = theClass;
        String packagePath = null;
        String packageName = ClassNameInfo.getQualifier(qualifiedClassName);
        if (packageName == null) {
            packageName = "";
        } else if (packageName.length() > 0) {
            packagePath = packageName.replace('.', File.separatorChar);
        }
        
        // Do we have a root directory?
        if (rootDir != null) {
            
            // Yes, do we have a package name?
            if (packagePath != null) {
                
                // Yes, so use it as the root. Open the directory...
                outputDir = new File(rootDir, packagePath);
                
                // Make sure the directory exists...
                ensureDirectory(outputDir,env);
            } else {
                
                // Default package, so use root as output dir...
                outputDir = rootDir;
            }
        } else {
            
            // No root directory. Get the current working directory...
            String workingDirPath = System.getProperty("user.dir");
            File workingDir = new File(workingDirPath);
            
            // Do we have a package name?
            if (packagePath == null) {
                
                // No, so use working directory...
                outputDir = workingDir;
            } else {
                
                // Yes, so use working directory as the root...
                outputDir = new File(workingDir, packagePath);
                
                // Make sure the directory exists...
                ensureDirectory(outputDir,env);
            }
        }
        
        // Finally, return the directory...
        return outputDir;
    }
    
    private static void ensureDirectory(File dir, ProcessorEnvironment env)
        throws GeneratorException {
            
        if (!dir.exists()) {
            dir.mkdirs();
            if (!dir.exists()) {
                throw new GeneratorException("generator.cannot.create.dir",
                    dir.getAbsolutePath());
            }
        }
    }
}

